package bin;

import kernel.service.ServiceManager;
import lib.CLIAppBase;

using tink.CoreApi;

@:build(macros.Binstore.includeBin("Service", ["srv"]))
class Service extends CLIAppBase {
	public function new() {
		registerSyncSubcommand("start", (args) -> {
			var result = ServiceManager.start(args.getString("name"));
			return handleResult(result);
		}, [String("name")]);

		registerSyncSubcommand("stop", (args) -> {
			var result = ServiceManager.stop(args.getString("name"));
			return handleResult(result);
		}, [String("name")]);

		registerSyncSubcommand("register", (args) -> {
			var name = args.getString("name");
			var binName = args.getString("binary");
			var rest = args.getRest();

			var result = ServiceManager.register(name, binName, rest);
			return handleResult(result);
		}, [String("name"), String("binary"), Rest("args")]);

		registerSyncSubcommand("unregister", (args) -> {
			var result = ServiceManager.unregister(args.getString("name"));
			return handleResult(result);
		}, [String("name")]);

		registerSyncSubcommand("list", (args) -> {
			var list = ServiceManager.listRunning();

			for (name in list) {
				this.handle.writeLine(name);
			}

			return true;
		});

		registerSyncSubcommand("enable", (args) -> {
			ServiceManager.enable(args.getString("name"));
			return true;
		}, [String("name")]);
	}

	private function handleResult(res:Outcome<Noise, String>):Bool {
		switch (res) {
			case Success(_):
				return true;
			case Failure(e):
				this.handle.write(e);
				return false;
		}
	}
}
