package bin.ns;

import lib.KVStore;
import haxe.ds.StringMap;
import kernel.net.Package.NetworkID;
import bin.ns.NameSystemRPC.INameSystemRPC;
import kernel.ps.ProcessHandle;
import kernel.ps.IProcess;

using Lambda;
using tink.CoreApi;

@:build(macros.Binstore.includeBin("NameSystem", ["ns", "namesystem"]))
class NameSystem implements IProcess implements INameSystemRPC {
	private var handle:ProcessHandle;

	private var idRecords:StringMap<NetworkID> = new StringMap();

	public function new() {}

	public function run(handle:ProcessHandle) {
		this.handle = handle;

		this.load();

		kernel.net.Net.registerProto("ns", (p) -> {
			NameSystemRPC.handlePackage(this, p);
		});
	}

	private function load() {
		var kv = new KVStore("NameSystem");
		kv.load();

		this.idRecords = kv.get("idRecords", new StringMap());
	}

	private function save() {
		var kv = new KVStore("NameSystem");

		kv.set("idRecords", this.idRecords);

		kv.save();
	}

	public function setIDRecord(name:String, id:NetworkID):Noise {
		this.idRecords.set(name, id);
		this.save();
		return Noise;
	}

	public function getIDRecord(name:String):Null<NetworkID> {
		return this.idRecords.get(name);
	}

	public function removeIDRecord(name:String):Noise {
		this.idRecords.remove(name);
		this.save();
		return Noise;
	}

	public function listIDRecords():Array<{name:String, id:NetworkID}> {
		var rtn = [];

		for (k => v in this.idRecords) {
			rtn.push({name: k, id: v});
		}

		return rtn;
	}
}
